# Copyright 2014 hm authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import unittest

from hm import managers, storage
from hm.model.host import Host
from mock import patch, call


class FakeManager(managers.BaseManager):
    def __init__(self, config=None):
        super(FakeManager, self).__init__(config)

    def create_host(self, name=None, alternative_id=0):
        host_id = self.get_conf('HOST_ID')
        return Host(id=host_id, dns_name="{}.{}.com".format(host_id, name), alternative_id=alternative_id)

    def destroy_host(self, id):
        if id == "explode":
            raise Exception("failure to destroy")

    def restore_host(self, id):
        if id == "explode":
            raise Exception("failure to restore")


managers.register('fake', FakeManager)


class HostTestCase(unittest.TestCase):

    def setUp(self):
        storage.MongoDBStorage()._hosts_collection().remove()

    def test_create(self):
        conf = {"HOST_ID": "fake-id"}
        host = Host.create('fake', 'my-group', conf)
        self.assertEqual(host.id, "fake-id")
        self.assertEqual(host.dns_name, "fake-id.my-group.com")
        self.assertEqual(host.manager, "fake")
        self.assertEqual(host.group, "my-group")
        self.assertEqual(host.config, conf)
        self.assertEqual(host.alternative_id, 0)
        db_host = Host.find('fake-id', conf=conf)
        self.assertEqual(db_host.id, "fake-id")
        self.assertEqual(db_host.dns_name, "fake-id.my-group.com")
        self.assertEqual(db_host.manager, "fake")
        self.assertEqual(db_host.group, "my-group")
        self.assertEqual(db_host.config, conf)
        self.assertEqual(db_host.alternative_id, 0)

    def test_create_alternatives(self):
        conf = {"HM_ALTERNATIVE_CONFIG_COUNT": "3"}
        conf.update({"HOST_ID": "fake-1"})
        host = Host.create('fake', 'my-group', conf)
        self.assertEqual(host.alternative_id, 0)
        conf.update({"HOST_ID": "fake-2"})
        host = Host.create('fake', 'my-group', conf)
        self.assertEqual(host.alternative_id, 1)
        conf.update({"HOST_ID": "fake-3"})
        host = Host.create('fake', 'my-group', conf)
        self.assertEqual(host.alternative_id, 2)
        conf.update({"HOST_ID": "fake-4"})
        host = Host.create('fake', 'my-group', conf)
        self.assertEqual(host.alternative_id, 0)
        conf.update({"HOST_ID": "fake-5"})
        host = Host.create('fake', 'my-group', conf)
        self.assertEqual(host.alternative_id, 1)
        conf.update({"HOST_ID": "fake-6"})
        host = Host.create('fake', 'another-group', conf)
        self.assertEqual(host.alternative_id, 0)
        hosts = Host.list(filters={'alternative_id': 0})
        self.assertEqual(len(hosts), 3)
        hosts = Host.list(filters={'alternative_id': 1})
        self.assertEqual(len(hosts), 2)
        hosts = Host.list(filters={'alternative_id': 2})
        self.assertEqual(len(hosts), 1)

    def test_destroy(self):
        host = Host.create('fake', 'my-group', {"HOST_ID": "fake-id"})
        self.assertEqual(host.id, "fake-id")
        host.destroy()
        db_host = Host.find('fake-id')
        self.assertIsNone(db_host)

    @patch("hm.log.error")
    def test_destroy_ignores_manager_error(self, log):
        host = Host.create('fake', 'my-group', {"HOST_ID": "explode"})
        self.assertEqual(host.id, "explode")
        host.destroy()
        self.assertEqual(log.call_args, call("Error trying to destroy host 'explode' "
                                             "in 'fake': failure to destroy"))
        db_host = Host.find('explode')
        self.assertIsNone(db_host)

    def test_restore(self):
        host = Host.create('fake', 'my-group', {"HOST_ID": "fake-id"})
        self.assertEqual(host.id, "fake-id")
        host.restore()

    @patch("hm.log.error")
    def test_restore_log_and_raises_exception_on_error(self, log):
        host = Host.create('fake', 'my-group', {"HOST_ID": "explode"})
        self.assertEqual(host.id, "explode")
        self.assertRaises(Exception, host.restore)
        self.assertEqual(log.call_args, call("Error trying to restore host 'explode' "
                                             "in 'fake': failure to restore"))
        db_host = Host.find('explode')
        self.assertEqual(db_host.id, "explode")

    def test_list(self):
        h1 = Host.create('fake', 'my-group1', {"HOST_ID": "fake-id-1"})
        h2 = Host.create('fake', 'my-group1', {"HOST_ID": "fake-id-2"})
        h3 = Host.create('fake', 'my-group2', {"HOST_ID": "fake-id-3"})
        conf = {'MY_CONF': 1}
        hosts = Host.list(conf=conf)
        self.assertDictEqual(hosts[0].config, conf)
        self.assertDictEqual(hosts[1].config, conf)
        self.assertDictEqual(hosts[2].config, conf)
        self.assertItemsEqual([h.to_json() for h in hosts], [h1.to_json(), h2.to_json(), h3.to_json()])
        hosts = Host.list({'group': 'my-group1'})
        self.assertItemsEqual([h.to_json() for h in hosts], [h1.to_json(), h2.to_json()])

    def test_storage_use_database_conf(self):
        storage.MongoDBStorage({"MONGO_DATABASE": "alternative_host_manager"})._hosts_collection().remove()
        h1 = Host.create('fake', 'my-group1', {
            "HOST_ID": "fake-id-x", "MONGO_DATABASE": "alternative_host_manager"
        })
        stor = h1.storage()
        self.assertEqual(stor.mongo_database, "alternative_host_manager")
        self.assertEqual(stor.db.name, "alternative_host_manager")
        h1.destroy()
        db_host = Host.find('fake-id-x')
        self.assertIsNone(db_host)

    def test_storage_use_uri_conf(self):
        conf = {
            "DBAAS_MONGODB_ENDPOINT": "mongodb://127.0.0.1:27017/some_other_db",
            "MONGO_URI": "mongodb://127.0.0.1:27017/ignored",
        }
        storage.MongoDBStorage(conf)._hosts_collection().remove()
        host_config = {
            "HOST_ID": "fake-id-x"
        }
        host_config.update(conf)
        h1 = Host.create('fake', 'my-group1', host_config)
        stor = h1.storage()
        self.assertEqual(stor.mongo_database, "some_other_db")
        self.assertEqual(stor.db.name, "some_other_db")
        h1.destroy()
        db_host = Host.find('fake-id-x')
        self.assertIsNone(db_host)
        conf = {
            "MONGO_URI": "mongodb://127.0.0.1:27017/now_used",
        }
        storage.MongoDBStorage(conf)._hosts_collection().remove()
        host_config = {
            "HOST_ID": "fake-id-x"
        }
        host_config.update(conf)
        h1 = Host.create('fake', 'my-group1', host_config)
        stor = h1.storage()
        self.assertEqual(stor.mongo_database, "now_used")
        self.assertEqual(stor.db.name, "now_used")
        h1.destroy()
        db_host = Host.find('fake-id-x')
        self.assertIsNone(db_host)



# Copyright 2015 hm authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from hm import managers
from hm.model import host
from hm.iaas.cloudstack_client import CloudStack


class CloudStackManager(managers.BaseManager):

    def __init__(self, config=None):
        super(CloudStackManager, self).__init__(config)
        url = self.get_conf("CLOUDSTACK_API_URL")
        key = self.get_conf("CLOUDSTACK_API_KEY")
        secret_key = self.get_conf("CLOUDSTACK_SECRET_KEY")
        self.client = CloudStack(url, key, secret_key)

    def create_host(self, name=None, alternative_id=0):
        group = self.get_conf("CLOUDSTACK_GROUP", "")
        user_data = self.get_user_data()
        if group and name:
            name = "{}_{}".format(group, name)
        elif not name:
            name = group
        data = {
            "group": group,
            "displayname": name,
            "templateid": self._get_alternate_conf("CLOUDSTACK_TEMPLATE_ID", alternative_id),
            "zoneid": self._get_alternate_conf("CLOUDSTACK_ZONE_ID", alternative_id),
            "serviceofferingid": self._get_alternate_conf("CLOUDSTACK_SERVICE_OFFERING_ID", alternative_id),
        }
        if user_data:
            data["userdata"] = self.client.encode_user_data(user_data)
        project_id = self._get_alternate_conf("CLOUDSTACK_PROJECT_ID", alternative_id, None)
        if project_id:
            data["projectid"] = project_id
        network_ids = self._get_alternate_conf("CLOUDSTACK_NETWORK_IDS", alternative_id, None)
        if network_ids:
            data["networkids"] = network_ids
        vm_job = self.client.deployVirtualMachine(data)
        max_tries = int(self.get_conf("CLOUDSTACK_MAX_TRIES", 100))
        if not vm_job.get("jobid"):
            raise CloudStackException(
                "unexpected response from deployVirtualMachine({}), expected jobid key, got: {}".format(
                    repr(data), repr(vm_job)))
        vm = self._wait_for_unit(vm_job, max_tries, project_id)
        tags = self.get_conf("HOST_TAGS", "")
        if tags:
            self._tag_vm(tags.split(","), vm, project_id)
        return host.Host(id=vm["id"], dns_name=self._get_dns_name(vm), alternative_id=alternative_id)

    def _tag_vm(self, tag_list, vm, project_id=None):
        params = {}

        for i, tag in enumerate(tag_list, start=1):
            parts = tag.split(":", 2)
            if len(parts) < 2:
                continue
            key, value = parts
            params["tags[{}].key".format(i)] = key
            params["tags[{}].value".format(i)] = value

        if not params:
            return

        params.update({"resourcetype": "UserVm", "resourceids": vm["id"]})
        if project_id:
            params["projectid"] = project_id
        self.client.createTags(params)

    def destroy_host(self, host_id):
        self.client.destroyVirtualMachine({"id": host_id})

    def restore_host(self, host_id):
        self.client.make_request('restoreVirtualMachine', {'virtualmachineid': host_id},
                                 response_key='restorevmresponse')

    def _get_dns_name(self, vm):
        if not vm.get("nic"):
            return ""
        network_index = int(self.get_conf("CLOUDSTACK_PUBLIC_NETWORK_INDEX", 0))
        dns_name = vm["nic"][network_index]["ipaddress"]
        return dns_name

    def _wait_for_unit(self, vm_job, max_tries, project_id):
        self.client.wait_for_job(vm_job["jobid"], max_tries)
        data = {"id": vm_job["id"]}
        if project_id:
            data["projectid"] = project_id
        vms = self.client.listVirtualMachines(data)
        return vms["virtualmachine"][0]

    def _get_alternate_conf(self, name, alternative_id, default=None):
        env_var = "{}_{}".format(name, alternative_id)
        val = self.get_conf(env_var, default)
        if val is not None:
            return val
        return self.get_conf(name, default)


class CloudStackException(Exception):
    pass


managers.register('cloudstack', CloudStackManager)



# Copyright 2014 hm authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from hm import config


_managers = {}
_expected_methods = ['create_load_balancer', 'destroy_load_balancer', 'attach_real', 'detach_real']


def register(name, cls):
    for m in _expected_methods:
        if not getattr(cls, m, None):
            raise InvalidLBManager("Expected method '{}' not found in {}".format(m, cls))
    _managers[name] = cls


def by_name(name, conf=None):
    return _managers[name](conf)


class BaseLBManager(object):
    def __init__(self, conf):
        self.config = conf or {}

    def get_conf(self, name, default=config.undefined):
        return config.get_config(name, default, self.config)


class LBConfig(object):
    environment_p44 = None
    client = None
    finality = None
    healthcheck = None
    healthcheck_expect = None
    cache = None
    lb_method = None
    persistence = None
    maxconn = None

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            self.__setattr__(k, v)


class InvalidLBManager(Exception):
    pass



# Copyright 2014 hm authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import os


undefined = object()


def get_config(name, default=undefined, config=None):
    try:
        value = (config or {}).get(name)
        if value is None:
            value = os.environ[name]
        return value
    except KeyError:
        if default is not undefined:
            return default
        raise MissConfigurationError("env var {} is required".format(name))


class MissConfigurationError(Exception):
    pass



# Copyright 2014 hm authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from hm import lb_managers, log
from hm.model import load_balancer
from hm.iaas.cloudstack_client import CloudStack

network_api_available = True
try:
    from networkapiclient import EnvironmentVIP, Ip, Vip
    from networkapiclient.exception import IpNaoExisteError
except ImportError:
    network_api_available = False


class NetworkApiCloudstackLB(lb_managers.BaseLBManager):

    def __init__(self, config=None):
        super(NetworkApiCloudstackLB, self).__init__(config)
        url = self.get_conf("CLOUDSTACK_API_URL")
        key = self.get_conf("CLOUDSTACK_API_KEY")
        secret_key = self.get_conf("CLOUDSTACK_SECRET_KEY")
        self.cs_client = CloudStack(url, key, secret_key)

        self.networkapi_endpoint = self.get_conf("NETWORKAPI_ENDPOINT")
        self.networkapi_user = self.get_conf("NETWORKAPI_USER")
        self.networkapi_password = self.get_conf("NETWORKAPI_PASSWORD")
        self.create_project_id = self.get_conf("CLOUDSTACK_PROJECT_ID", None)
        self.vip_network_index = int(self.get_conf("CLOUDSTACK_VIP_NETWORK_INDEX", 0))
        self.vip_config = VIPConfig(
            environment_p44=self.get_conf("NETWORKAPI_AMBIENTE_P44_TXT"),
            client=self.get_conf("NETWORKAPI_CLIENTE_TXT"),
            finality=self.get_conf("NETWORKAPI_FINALIDADE_TXT"),
            healthcheck=self.get_conf("VIP_HEALTHCHECK", "GET / HTTP/1.1\r\n\r\n"),
            healthcheck_expect=self.get_conf("VIP_HEALTHCHECK_EXPECT", 26),
            lb_method=self.get_conf("VIP_METHOD_BAL", "least-conn"),
            persistence=self.get_conf("VIP_PERSISTENCE", "(nenhum)"),
            cache=self.get_conf("VIP_CACHE", "(nenhum)"),
            maxconn=self.get_conf("VIP_MAXCONN", "1000"),
            business_area=self.get_conf("VIP_BUSINESS_AREA", ""),
            service_name=self.get_conf("VIP_SERVICE_NAME", ""),
            port_mapping=self.get_conf("VIP_PORT_MAPPING").split(',')
        )

    def create_load_balancer(self, name):
        vip_id, ip_id, address = self._create_vip(name, self.vip_config)

        return load_balancer.LoadBalancer(
            vip_id, name, address,
            ip_id=ip_id,
            project_id=self.create_project_id)

    def destroy_load_balancer(self, lb):
        data = {
            "vipid": lb.id
        }
        if hasattr(lb, 'project_id'):
            data["projectid"] = lb.project_id
        self.cs_client.removeGloboNetworkVip(data)
        return self._remove_vip(lb)

    def attach_real(self, lb, host):
        real_data, network_data = self._get_association_data(lb, host)
        self.cs_client.addGloboNetworkVipToAccount(network_data)
        self.cs_client.associateGloboNetworkRealToVip(real_data)

    def detach_real(self, lb, host):
        real_data, _ = self._get_association_data(lb, host)
        self.cs_client.disassociateGloboNetworkRealFromVip(real_data)

    def _get_association_data(self, lb, host):
        real_data = {"vipid": lb.id}
        network_data = {"vipid": lb.id}
        list_data = {"id": host.id}

        if getattr(lb, 'project_id', None):
            list_data["projectid"] = \
                real_data["projectid"] = \
                network_data["projectid"] = lb.project_id
        vms = self.cs_client.listVirtualMachines(list_data)["virtualmachine"]
        nic = vms[0]["nic"][self.vip_network_index]

        real_data["nicid"] = nic["id"]
        network_data["networkid"] = nic["networkid"]
        return real_data, network_data

    def _remove_vip(self, lb):
        client_vip = Vip.Vip(
            self.networkapi_endpoint, self.networkapi_user,
            self.networkapi_password)
        client_vip.remove_script(lb.id)
        client_vip.remover(lb.id)
        try:
            client_ip = Ip.Ip(self.networkapi_endpoint, self.networkapi_user,
                              self.networkapi_password)
            client_ip.delete_ip4(lb.ip_id)
        except IpNaoExisteError:
            pass

    def _create_vip(self, name, vip_config):
        vip_id = None
        client_evip = EnvironmentVIP.EnvironmentVIP(
            self.networkapi_endpoint,
            self.networkapi_user,
            self.networkapi_password)
        evip = client_evip.search(ambiente_p44_txt=vip_config.environment_p44,
                                  cliente_txt=vip_config.client,
                                  finalidade_txt=vip_config.finality)
        client_ip = Ip.Ip(self.networkapi_endpoint, self.networkapi_user,
                          self.networkapi_password)
        vip_ip = client_ip.get_available_ip4_for_vip(evip["environment_vip"]["id"],
                                                     u"tsuru hm {0}".format(name))
        try:
            client_vip = Vip.Vip(self.networkapi_endpoint, self.networkapi_user,
                                 self.networkapi_password)
            request = client_vip.add(id_ipv4=vip_ip["ip"]["id"],
                                     id_ipv6=None,
                                     id_healthcheck_expect=vip_config.healthcheck_expect,
                                     finality=vip_config.finality,
                                     client=vip_config.client,
                                     environment=vip_config.environment_p44,
                                     cache=vip_config.cache,
                                     method_bal=vip_config.lb_method,
                                     persistence=vip_config.persistence,
                                     healthcheck_type=u"HTTP",
                                     healthcheck=vip_config.healthcheck,
                                     timeout="5",
                                     host="{0}.hm.tsuru".format(name),
                                     maxcon=vip_config.maxconn,
                                     areanegocio=vip_config.business_area,
                                     nome_servico=vip_config.service_name,
                                     l7_filter=None,
                                     reals=[],
                                     reals_prioritys=[],
                                     reals_weights=[],
                                     ports=vip_config.port_mapping)
            vip_id = request["requisicao_vip"]["id"]
            log.debug(u"VIP request %s successfully created." % vip_id)
            client_vip.validate(vip_id)
            client_vip.criar(vip_id)
            address = "{oct1}.{oct2}.{oct3}.{oct4}".format(**vip_ip["ip"])
            return vip_id, vip_ip["ip"]["id"], address
        except Exception as e:
            if vip_id:
                try:
                    client_vip.remove_script(vip_id)
                    client_vip.remover(vip_id)
                except Exception as exc:
                    args = [str(a) for a in exc.args]
                    error = "".join(args)
                    log.error("Failed to remove the VIP {0}: {1}".format(vip_id, error))
            client_ip.delete_ip4(vip_ip["ip"]["id"])
            raise e


class VIPConfig(object):
    environment_p44 = None
    client = None
    finality = None
    healthcheck = None
    healthcheck_expect = None
    cache = None
    lb_method = None
    persistence = None
    maxconn = None
    business_area = None
    service_name = None
    port_mapping = None

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            self.__setattr__(k, v)


if network_api_available:
    lb_managers.register('networkapi_cloudstack', NetworkApiCloudstackLB)



# Copyright 2014 hm authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import unittest

import mock

from hm import managers, config


class BaseManagerTestCase(unittest.TestCase):

    def test_get_user_data_txt(self):
        manager = managers.BaseManager({
            'USER_DATA_TXT': 'my user data'
        })
        data = manager.get_user_data()
        self.assertEqual(data, "my user data")

    @mock.patch('hm.managers.requests')
    def test_get_user_data_url(self, requests):
        manager = managers.BaseManager({
            'USER_DATA_URL': 'http://localhost/somewhere'
        })
        requests.get.return_value.status_code = 200
        requests.get.return_value.text = "user data from url"
        data = manager.get_user_data()
        self.assertEqual(data, "user data from url")
        requests.get.assert_called_with("http://localhost/somewhere")

    @mock.patch('hm.managers.requests')
    def test_get_user_data_url_error(self, requests):
        manager = managers.BaseManager({
            'USER_DATA_URL': 'http://localhost/somewhere'
        })
        requests.get.return_value.status_code = 500
        requests.get.return_value.text = "my error"
        with self.assertRaises(config.MissConfigurationError) as rsp:
            manager.get_user_data()
        self.assertEqual(str(rsp.exception),
                         "invalid user data response from http://localhost/somewhere: 500 - my error")



# Copyright 2014 hm authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from hm import lb_managers, log, model
from hm.model.host import Host


class LoadBalancer(model.BaseModel):

    def __init__(self, id, name, address, conf=None, **kwargs):
        self.id = id
        self.name = name
        self.address = address
        self.manager = None
        self.extra_args = set()
        self.config = conf
        self.hosts = []
        for k, v in kwargs.items():
            self.extra_args.add(k)
            setattr(self, k, v)

    def to_json(self):
        obj = {
            '_id': self.name,
            'id': self.id,
            'address': self.address,
            'manager': self.manager,
        }
        for key in self.extra_args:
            obj[key] = getattr(self, key)
        return obj

    @classmethod
    def from_dict(cls, dict, conf=None):
        if dict is None:
            return None
        dict['name'] = dict['_id']
        del dict['_id']
        dict['conf'] = conf
        hosts_data = dict.get('hosts', None)
        if hosts_data:
            dict['hosts'] = [Host.from_dict(h, conf=conf) for h in hosts_data]
        return cls(**dict)

    @classmethod
    def create(cls, manager_name, name, conf=None):
        manager = lb_managers.by_name(manager_name, conf)
        lb = manager.create_load_balancer(name)
        lb.manager = manager_name
        lb.config = conf
        model.storage(conf).store_load_balancer(lb)
        return lb

    @classmethod
    def find(cls, name, conf=None):
        return model.storage(conf).find_load_balancer(name)

    @classmethod
    def list(cls, filters=None, conf=None):
        return model.storage(conf).list_load_balancers(filters)

    def destroy(self):
        manager = self._manager()
        try:
            manager.destroy_load_balancer(self)
        except Exception as e:
            log.error("Error trying to destroy load balancer name: '{}' id: '{}' in '{}': {}".format(
                self.name, self.id, self.manager, e))
        self.storage().remove_load_balancer(self.name)

    def add_host(self, host):
        manager = self._manager()
        manager.attach_real(self, host)
        self.storage().add_host_to_load_balancer(self.name, host)
        self.hosts.append(host)

    def remove_host(self, host):
        manager = self._manager()
        manager.detach_real(self, host)
        self.storage().remove_host_from_load_balancer(self.name, host)
        self.hosts = [h for h in self.hosts if h.id != host.id]

    def _manager(self):
        return lb_managers.by_name(self.manager, self.config)



# coding: utf-8
import unittest

import mock

from hm.lb_managers import cloudstack
from hm.model import load_balancer, host


class CloudstackLBTestCase(unittest.TestCase):

    def setUp(self):
        self.conf = {
            'CLOUDSTACK_API_URL': 'http://localhost',
            'CLOUDSTACK_API_KEY': 'key',
            'CLOUDSTACK_SECRET_KEY': 'secret',
            'CLOUDSTACK_PROJECT_ID': 'proj-123',

            'CLOUDSTACK_LB_ASSOCIATE_IP_COMMAND': 'associateIpAddress',
            'CLOUDSTACK_LB_DISASSOCIATE_IP_COMMAND': 'disassociateIpAddress',
            'CLOUDSTACK_LB_ASSIGN_NETWORK_COMMAND': 'assignNetwork',
            'CLOUDSTACK_LB_NETWORK_ID': 'net-id',
            'CLOUDSTACK_LB_ZONE_ID': 'zone-id',
            'CLOUDSTACK_LB_VPC_ID': 'vpc-id',
            'CLOUDSTACK_LB_ENVIRONMENT_ID': 'env-id',
            'CLOUDSTACK_LB_ALGORITHM': 'leastconn',
            'CLOUDSTACK_LB_PORT_MAPPING': '80:8080,81:8081,82:8082',
            'CLOUDSTACK_LB_OPEN_FIREWALL': 'false',
            'CLOUDSTACK_LB_HEALTHCHECK': 'GET /',
            'CLOUDSTACK_LB_NETWORK_INDEX': '0',
            'CLOUDSTACK_LB_DOMAIN': 'abc.com',
            'CLOUDSTACK_LB_NETWORK_ID_0': 'net-id',
            'CLOUDSTACK_LB_NETWORK_ID_1': 'net-id2',
            'CLOUDSTACK_LB_NETWORK_ID_2': 'net-id3',
        }

    def test_init(self):
        manager = cloudstack.CloudstackLB(self.conf)
        self.assertEqual(manager.cs_client.api_url, 'http://localhost')
        self.assertEqual(manager.cs_client.api_key, 'key')
        self.assertEqual(manager.cs_client.secret, 'secret')
        self.assertEqual(manager.project_id, 'proj-123')
        self.assertEqual(manager.associate_ip_command, 'associateIpAddress')
        self.assertEqual(manager.disassociate_ip_command, 'disassociateIpAddress')
        self.assertEqual(manager.assign_network_command, 'assignNetwork')
        self.assertEqual(manager.lb_network_id, 'net-id')
        self.assertEqual(manager.lb_zone_id, 'zone-id')
        self.assertEqual(manager.lb_vpc_id, 'vpc-id')
        self.assertEqual(manager.lb_environment_id, 'env-id')
        self.assertEqual(manager.lb_algorithm, 'leastconn')
        self.assertEqual(manager.lb_port_mapping, '80:8080,81:8081,82:8082')
        self.assertEqual(manager.lb_open_firewall, 'false')
        self.assertEqual(manager.lb_healthcheck, 'GET /')
        self.assertEqual(manager.lb_network_index, 0)
        self.assertEqual(manager.lb_domain, 'abc.com')

    @mock.patch("hm.lb_managers.cloudstack.CloudStack")
    def test_create_load_balancer(self, cs_mock):
        cs_instance = cs_mock.return_value
        cs_instance.make_request.return_value = {'id': 'lb-ip-id', 'jobid': 'j1'}
        cs_instance.createLoadBalancerRule.return_value = {'id': 'lb-id', 'jobid': 'j2'}

        def wait_mock(jid, tries):
            if jid == 'j1':
                return {'jobresult': True}
            if jid == 'j2':
                return {'jobresult': {'loadbalancer': {'publicip': '192.168.1.5'}}}
        cs_instance.wait_for_job.side_effect = wait_mock
        manager = cloudstack.CloudstackLB(self.conf)
        lb = manager.create_load_balancer("tsuru")
        self.assertEqual('lb-id', lb.id)
        self.assertEqual('lb-ip-id', lb.ip_id)
        self.assertEqual("192.168.1.5", lb.address)
        self.assertEqual("tsuru", lb.name)
        self.assertEqual("proj-123", lb.project_id)
        cs_instance.make_request.assert_called_once_with('associateIpAddress', {
            'networkid': 'net-id',
            'projectid': 'proj-123',
            'zoneid': 'zone-id',
            'vpcid': 'vpc-id',
            'lbenvironmentid': 'env-id',
        }, response_key='associateipaddressresponse')
        cs_instance.createLoadBalancerRule.assert_called_once_with({
            'networkid': 'net-id',
            'algorithm': 'leastconn',
            'publicport': '80',
            'privateport': '8080',
            'openfirewall': 'false',
            'publicipid': 'lb-ip-id',
            'name': 'tsuru.abc.com',
            'additionalportmap': '81:8081,82:8082',
            'projectid': 'proj-123',
        })
        cs_instance.assignNetworksToLoadBalancerRule.assert_called_once_with({
            'id': 'lb-id',
            'projectid': 'proj-123',
            'networkids': 'net-id2,net-id3',
        })

    @mock.patch("hm.lb_managers.cloudstack.CloudStack")
    def test_create_load_balancer_rollback_ip(self, cs_mock):
        cs_instance = cs_mock.return_value
        cs_instance.make_request.return_value = {'id': 'lb-ip-id', 'jobid': 'j1'}

        def exc(*args):
            raise Exception("some error")
        cs_instance.createLoadBalancerRule.side_effect = exc
        cs_instance.wait_for_job.return_value = {'jobresult': True}
        manager = cloudstack.CloudstackLB(self.conf)
        with self.assertRaises(Exception) as exc_data:
            manager.create_load_balancer("tsuru")
        self.assertEqual(str(exc_data.exception), 'some error')
        cs_instance.make_request.assert_any_call('associateIpAddress', {
            'networkid': 'net-id',
            'projectid': 'proj-123',
            'zoneid': 'zone-id',
            'vpcid': 'vpc-id',
            'lbenvironmentid': 'env-id',
        }, response_key='associateipaddressresponse')
        cs_instance.make_request.assert_any_call('disassociateIpAddress', {
            'projectid': 'proj-123',
            'id': 'lb-ip-id',
        })
        cs_instance.createLoadBalancerRule.assert_called_with({
            'networkid': 'net-id',
            'algorithm': 'leastconn',
            'publicport': '80',
            'privateport': '8080',
            'openfirewall': 'false',
            'publicipid': 'lb-ip-id',
            'name': 'tsuru.abc.com',
            'additionalportmap': '81:8081,82:8082',
            'projectid': 'proj-123',
        })

    @mock.patch("hm.lb_managers.cloudstack.CloudStack")
    def test_attach_real(self, cs_mock):
        cs_instance = cs_mock.return_value
        cs_instance.make_request.return_value = {'jobid': 'j1'}
        cs_instance.assignToLoadBalancerRule.return_value = {'jobid': 'j2'}
        cs_instance.wait_for_job.return_value = {'jobresult': True}

        vms = {"virtualmachine": [{"id": "abc123", "nic": [{"id": "def456", "networkid": "netid1"}]}]}
        cs_instance.listVirtualMachines.return_value = vms

        lb = load_balancer.LoadBalancer('lbid', 'lbname', 'lbaddr', ip_id='ip_id', project_id='projid')
        h = host.Host('hostid', 'hostaddr')
        manager = cloudstack.CloudstackLB(self.conf)
        manager.attach_real(lb, h)
        cs_instance.make_request.assert_called_once_with('assignNetwork', {
            'id': 'lbid',
            'networkids': 'netid1',
            'projectid': 'projid',
        })
        cs_instance.assignToLoadBalancerRule.assert_called_once_with({
            'id': 'lbid',
            'projectid': 'projid',
            'virtualmachineids': 'hostid',
        })

    @mock.patch("hm.lb_managers.cloudstack.CloudStack")
    def test_detach_real(self, cs_mock):
        cs_instance = cs_mock.return_value
        cs_instance.removeFromLoadBalancerRule.return_value = {'jobid': 'j1'}
        lb = load_balancer.LoadBalancer('lbid', 'lbname', 'lbaddr', ip_id='ip_id', project_id='projid')
        h = host.Host('hostid', 'hostaddr')
        manager = cloudstack.CloudstackLB(self.conf)
        manager.detach_real(lb, h)
        cs_instance.removeFromLoadBalancerRule.assert_called_once_with({
            'id': 'lbid',
            'projectid': 'projid',
            'virtualmachineids': 'hostid',
        })
        cs_instance.wait_for_job.assert_called_with('j1', 100)

    @mock.patch("hm.lb_managers.cloudstack.CloudStack")
    def test_destroy_load_balancer(self, cs_mock):
        cs_instance = cs_mock.return_value
        cs_instance.deleteLoadBalancerRule.return_value = {'jobid': 'j1'}
        cs_instance.make_request.return_value = {'jobid': 'j2'}
        lb = load_balancer.LoadBalancer('lbid', 'lbname', 'lbaddr', ip_id='ipid', project_id='projid')
        manager = cloudstack.CloudstackLB(self.conf)
        manager.destroy_load_balancer(lb)
        cs_instance.deleteLoadBalancerRule.assert_called_once_with({
            'id': 'lbid',
            'projectid': 'projid',
        })
        cs_instance.make_request.assert_called_once_with('disassociateIpAddress', {
            'id': 'ipid',
            'projectid': 'projid',
        })
        cs_instance.wait_for_job.assert_any_call('j1', 100)
        cs_instance.wait_for_job.assert_any_call('j2', 100)



# Copyright 2015 hm authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import unittest

import mock

from hm import config
from hm.managers import cloudstack
from hm.iaas import cloudstack_client


class CloudStackManagerTestCase(unittest.TestCase):

    def setUp(self):
        self.config = {
            "CLOUDSTACK_API_URL": "http://cloudstackapi",
            "CLOUDSTACK_API_KEY": "key",
            "CLOUDSTACK_SECRET_KEY": "secret",
        }

    def test_init(self):
        client = cloudstack.CloudStackManager(self.config)
        self.assertEqual(client.client.api_url, self.config["CLOUDSTACK_API_URL"])
        self.assertEqual(client.client.api_key, self.config["CLOUDSTACK_API_KEY"])
        self.assertEqual(client.client.secret, self.config["CLOUDSTACK_SECRET_KEY"])

    def test_init_no_api_url(self):
        with self.assertRaises(config.MissConfigurationError) as cm:
            cloudstack.CloudStackManager()
        exc = cm.exception
        self.assertEqual(("env var CLOUDSTACK_API_URL is required",),
                         exc.args)

    def test_init_no_api_key(self):
        with self.assertRaises(config.MissConfigurationError) as cm:
            cloudstack.CloudStackManager({"CLOUDSTACK_API_URL": "something"})
        exc = cm.exception
        self.assertEqual(("env var CLOUDSTACK_API_KEY is required",),
                         exc.args)

    def test_init_no_secret_key(self):
        with self.assertRaises(config.MissConfigurationError) as cm:
            cloudstack.CloudStackManager({
                "CLOUDSTACK_API_URL": "something",
                "CLOUDSTACK_API_KEY": "not_secret",
            })
        exc = cm.exception
        self.assertEqual(("env var CLOUDSTACK_SECRET_KEY is required",),
                         exc.args)

    def test_create(self):
        self.config.update({
            "CLOUDSTACK_TEMPLATE_ID": "abc123",
            "CLOUDSTACK_SERVICE_OFFERING_ID": "qwe123",
            "CLOUDSTACK_ZONE_ID": "zone1",
            "CLOUDSTACK_PROJECT_ID": "project-123",
            "CLOUDSTACK_NETWORK_IDS": "net-123",
            "CLOUDSTACK_GROUP": "feaas",
        })

        client_mock = mock.Mock()
        client_mock.deployVirtualMachine.return_value = {"id": "abc123",
                                                         "jobid": "qwe321"}
        vm = {"id": "abc123", "nic": [{"ipaddress": "10.0.0.1"}]}
        client_mock.listVirtualMachines.return_value = {"virtualmachine": [vm]}

        manager = cloudstack.CloudStackManager(self.config)
        manager.client = client_mock
        host = manager.create_host('xxx')
        self.assertEqual("abc123", host.id)
        self.assertEqual("10.0.0.1", host.dns_name)
        create_data = {
            "group": "feaas",
            "displayname": "feaas_xxx",
            "templateid": "abc123",
            "zoneid": "zone1",
            "serviceofferingid": "qwe123",
            "networkids": "net-123",
            "projectid": "project-123",
        }
        client_mock.deployVirtualMachine.assert_called_with(create_data)
        client_mock.wait_for_job.assert_called_with('qwe321', 100)

    def test_create_with_tags(self):
        self.config.update({
            "CLOUDSTACK_TEMPLATE_ID": "abc123",
            "CLOUDSTACK_SERVICE_OFFERING_ID": "qwe123",
            "CLOUDSTACK_ZONE_ID": "zone1",
            "CLOUDSTACK_PROJECT_ID": "project-123",
            "CLOUDSTACK_NETWORK_IDS": "net-123",
            "CLOUDSTACK_GROUP": "feaas",
            "HOST_TAGS": "Name:something,monitor:1,wait=wat",
        })

        client_mock = mock.Mock()
        client_mock.deployVirtualMachine.return_value = {"id": "abc123",
                                                         "jobid": "qwe321"}
        vm = {"id": "abc123", "nic": [{"ipaddress": "10.0.0.1"}]}
        client_mock.listVirtualMachines.return_value = {"virtualmachine": [vm]}

        manager = cloudstack.CloudStackManager(self.config)
        manager.client = client_mock
        host = manager.create_host('xxx')
        self.assertEqual("abc123", host.id)
        self.assertEqual("10.0.0.1", host.dns_name)
        create_data = {
            "group": "feaas",
            "displayname": "feaas_xxx",
            "templateid": "abc123",
            "zoneid": "zone1",
            "serviceofferingid": "qwe123",
            "networkids": "net-123",
            "projectid": "project-123",
        }
        client_mock.deployVirtualMachine.assert_called_with(create_data)
        client_mock.wait_for_job.assert_called_with('qwe321', 100)
        tag_data = {
            "resourcetype": "UserVm",
            "resourceids": "abc123",
            "projectid": "project-123",
            "tags[1].key": "Name",
            "tags[1].value": "something",
            "tags[2].key": "monitor",
            "tags[2].value": "1",
        }
        client_mock.createTags.assert_called_with(tag_data)

    def test_create_no_group(self):
        self.config.update({
            "CLOUDSTACK_TEMPLATE_ID": "abc123",
            "CLOUDSTACK_SERVICE_OFFERING_ID": "qwe123",
            "CLOUDSTACK_ZONE_ID": "zone1",
            "CLOUDSTACK_PROJECT_ID": "project-123",
            "CLOUDSTACK_NETWORK_IDS": "net-123",
        })

        client_mock = mock.Mock()
        client_mock.deployVirtualMachine.return_value = {"id": "abc123",
                                                         "jobid": "qwe321"}
        vm = {"id": "abc123", "nic": [{"ipaddress": "10.0.0.1"}]}
        client_mock.listVirtualMachines.return_value = {"virtualmachine": [vm]}

        manager = cloudstack.CloudStackManager(self.config)
        manager.client = client_mock
        host = manager.create_host('xxx')
        self.assertEqual("abc123", host.id)
        self.assertEqual("10.0.0.1", host.dns_name)
        create_data = {
            "group": "",
            "displayname": "xxx",
            "templateid": "abc123",
            "zoneid": "zone1",
            "serviceofferingid": "qwe123",
            "networkids": "net-123",
            "projectid": "project-123",
        }
        client_mock.deployVirtualMachine.assert_called_with(create_data)
        client_mock.wait_for_job.assert_called_with('qwe321', 100)

    def test_create_no_project_id(self):
        self.config.update({
            "CLOUDSTACK_TEMPLATE_ID": "abc123",
            "CLOUDSTACK_SERVICE_OFFERING_ID": "qwe123",
            "CLOUDSTACK_ZONE_ID": "zone1",
            "CLOUDSTACK_NETWORK_IDS": "net-123",
            "CLOUDSTACK_GROUP": "feaas",
        })

        client_mock = mock.Mock()
        client_mock.deployVirtualMachine.return_value = {"id": "abc123",
                                                         "jobid": "qwe321"}
        vm = {"id": "abc123", "nic": [{"ipaddress": "10.0.0.1"}]}
        client_mock.listVirtualMachines.return_value = {"virtualmachine": [vm]}

        manager = cloudstack.CloudStackManager(self.config)
        manager.client = client_mock
        host = manager.create_host()
        self.assertEqual("abc123", host.id)
        self.assertEqual("10.0.0.1", host.dns_name)
        create_data = {
            "group": "feaas",
            "displayname": "feaas",
            "templateid": "abc123",
            "zoneid": "zone1",
            "serviceofferingid": "qwe123",
            "networkids": "net-123",
        }
        client_mock.deployVirtualMachine.assert_called_with(create_data)
        client_mock.wait_for_job.assert_called_with('qwe321', 100)

    def test_create_no_network_id(self):
        self.config.update({
            "CLOUDSTACK_TEMPLATE_ID": "abc123",
            "CLOUDSTACK_SERVICE_OFFERING_ID": "qwe123",
            "CLOUDSTACK_ZONE_ID": "zone1",
            "CLOUDSTACK_GROUP": "feaas",
        })

        client_mock = mock.Mock()
        client_mock.deployVirtualMachine.return_value = {"id": "abc123",
                                                         "jobid": "qwe321"}
        vm = {"id": "abc123", "nic": [{"ipaddress": "10.0.0.1"}]}
        client_mock.listVirtualMachines.return_value = {"virtualmachine": [vm]}

        manager = cloudstack.CloudStackManager(self.config)
        manager.client = client_mock
        host = manager.create_host()
        self.assertEqual("abc123", host.id)
        self.assertEqual("10.0.0.1", host.dns_name)
        create_data = {
            "group": "feaas",
            "displayname": "feaas",
            "templateid": "abc123",
            "zoneid": "zone1",
            "serviceofferingid": "qwe123",
        }
        client_mock.deployVirtualMachine.assert_called_with(create_data)
        client_mock.wait_for_job.assert_called_with('qwe321', 100)

    def test_create_invalid_response(self):
        self.config.update({
            "CLOUDSTACK_TEMPLATE_ID": "abc123",
            "CLOUDSTACK_SERVICE_OFFERING_ID": "qwe123",
            "CLOUDSTACK_ZONE_ID": "zone1",
            "CLOUDSTACK_GROUP": "feaas",
        })

        client_mock = mock.Mock()
        client_mock.deployVirtualMachine.return_value = {"error": "xxx"}
        vm = {"id": "abc123", "nic": [{"ipaddress": "10.0.0.1"}]}
        client_mock.listVirtualMachines.return_value = {"virtualmachine": [vm]}

        manager = cloudstack.CloudStackManager(self.config)
        manager.client = client_mock
        with self.assertRaises(cloudstack.CloudStackException) as ctx:
            manager.create_host()
        exc = ctx.exception
        self.assertRegexpMatches(
            str(exc), r"unexpected response from deployVirtualMachine\({.+}\)"
            ", expected jobid key, got: {'error': 'xxx'}")
        create_data = {
            "group": "feaas",
            "displayname": "feaas",
            "templateid": "abc123",
            "zoneid": "zone1",
            "serviceofferingid": "qwe123",
        }
        client_mock.deployVirtualMachine.assert_called_with(create_data)

    def test_create_public_network_index(self):
        self.config.update({
            "CLOUDSTACK_TEMPLATE_ID": "abc123",
            "CLOUDSTACK_SERVICE_OFFERING_ID": "qwe123",
            "CLOUDSTACK_ZONE_ID": "zone1",
            "CLOUDSTACK_GROUP": "feaas",
            "CLOUDSTACK_PUBLIC_NETWORK_INDEX": "1",
        })

        client_mock = mock.Mock()
        client_mock.deployVirtualMachine.return_value = {"id": "abc123",
                                                         "jobid": "qwe321"}
        vm = {"id": "abc123", "nic": [{"ipaddress": "10.0.0.1"},
                                      {"ipaddress": "192.168.1.1"},
                                      {"ipaddress": "172.16.42.1"}]}
        client_mock.listVirtualMachines.return_value = {"virtualmachine": [vm]}

        manager = cloudstack.CloudStackManager(self.config)
        manager.client = client_mock
        host = manager.create_host()
        self.assertEqual("abc123", host.id)
        self.assertEqual("192.168.1.1", host.dns_name)
        create_data = {
            "group": "feaas",
            "displayname": "feaas",
            "templateid": "abc123",
            "zoneid": "zone1",
            "serviceofferingid": "qwe123",
        }
        client_mock.deployVirtualMachine.assert_called_with(create_data)
        client_mock.wait_for_job.assert_called_with('qwe321', 100)

    def test_create_public_multi_nic_no_network_index(self):
        self.config.update({
            "CLOUDSTACK_TEMPLATE_ID": "abc123",
            "CLOUDSTACK_SERVICE_OFFERING_ID": "qwe123",
            "CLOUDSTACK_ZONE_ID": "zone1",
            "CLOUDSTACK_GROUP": "feaas",
        })

        client_mock = mock.Mock()
        client_mock.deployVirtualMachine.return_value = {"id": "abc123",
                                                         "jobid": "qwe321"}
        vm = {"id": "abc123", "nic": [{"ipaddress": "10.0.0.1"},
                                      {"ipaddress": "192.168.1.1"},
                                      {"ipaddress": "172.16.42.1"}]}
        client_mock.listVirtualMachines.return_value = {"virtualmachine": [vm]}

        manager = cloudstack.CloudStackManager(self.config)
        manager.client = client_mock
        host = manager.create_host()
        self.assertEqual("abc123", host.id)
        self.assertEqual("10.0.0.1", host.dns_name)
        create_data = {
            "group": "feaas",
            "displayname": "feaas",
            "templateid": "abc123",
            "zoneid": "zone1",
            "serviceofferingid": "qwe123",
        }
        client_mock.deployVirtualMachine.assert_called_with(create_data)
        client_mock.wait_for_job.assert_called_with('qwe321', 100)

    def test_create_timeout(self):
        self.config.update({
            "CLOUDSTACK_TEMPLATE_ID": "abc123",
            "CLOUDSTACK_SERVICE_OFFERING_ID": "qwe123",
            "CLOUDSTACK_ZONE_ID": "zone1",
            "CLOUDSTACK_GROUP": "feaas",
            "CLOUDSTACK_MAX_TRIES": 1,
        })

        client_mock = mock.Mock()
        client_mock.deployVirtualMachine.return_value = {"id": "abc123",
                                                         "jobid": "qwe321"}
        vm = {"id": "abc123", "nic": [{"ipaddress": "10.0.0.1"}]}
        client_mock.listVirtualMachines.return_value = {"virtualmachine": [vm]}
        client_mock.wait_for_job.side_effect = cloudstack_client.MaxTryWaitingForJobError(1, 'qwe321')

        manager = cloudstack.CloudStackManager(self.config)
        manager.client = client_mock
        with self.assertRaises(cloudstack_client.MaxTryWaitingForJobError) as cm:
            manager.create_host()
        exc = cm.exception
        self.assertEqual(1, exc.max_tries)
        self.assertEqual("qwe321", exc.job_id)
        self.assertEqual("exceeded 1 tries waiting for job qwe321", str(exc))
        create_data = {
            "group": "feaas",
            "displayname": "feaas",
            "templateid": "abc123",
            "zoneid": "zone1",
            "serviceofferingid": "qwe123",
        }
        client_mock.wait_for_job.assert_called_with('qwe321', 1)
        client_mock.deployVirtualMachine.assert_called_with(create_data)

    def test_create_alternatives(self):
        self.config.update({
            "CLOUDSTACK_GROUP": "feaas",
            "CLOUDSTACK_TEMPLATE_ID": "template-base",
            "CLOUDSTACK_PROJECT_ID": "project-base",

            "CLOUDSTACK_ZONE_ID_0": "zone0",
            "CLOUDSTACK_NETWORK_IDS_0": "net0",
            "CLOUDSTACK_SERVICE_OFFERING_ID_0": "offering0",

            "CLOUDSTACK_ZONE_ID_1": "zone1",
            "CLOUDSTACK_NETWORK_IDS_1": "net1",
            "CLOUDSTACK_SERVICE_OFFERING_ID_1": "offering1",
            "CLOUDSTACK_PROJECT_ID_1": "project1",
        })

        client_mock = mock.Mock()
        client_mock.deployVirtualMachine.return_value = {"id": "abc123",
                                                         "jobid": "qwe321"}
        vm = {"id": "abc123", "nic": [{"ipaddress": "10.0.0.1"}]}
        client_mock.listVirtualMachines.return_value = {"virtualmachine": [vm]}

        manager = cloudstack.CloudStackManager(self.config)
        manager.client = client_mock
        host = manager.create_host('xxx', alternative_id=0)
        self.assertEqual("abc123", host.id)
        self.assertEqual("10.0.0.1", host.dns_name)
        create_data = {
            "group": "feaas",
            "displayname": "feaas_xxx",
            "templateid": "template-base",
            "zoneid": "zone0",
            "serviceofferingid": "offering0",
            "networkids": "net0",
            "projectid": "project-base",
        }
        client_mock.deployVirtualMachine.assert_called_with(create_data)
        client_mock.wait_for_job.assert_called_with('qwe321', 100)

        host = manager.create_host('xxx', alternative_id=1)
        create_data = {
            "group": "feaas",
            "displayname": "feaas_xxx",
            "templateid": "template-base",
            "zoneid": "zone1",
            "serviceofferingid": "offering1",
            "networkids": "net1",
            "projectid": "project1",
        }
        client_mock.deployVirtualMachine.assert_called_with(create_data)

    def test_destroy_host(self):
        manager = cloudstack.CloudStackManager(self.config)
        manager.client = mock.Mock()
        manager.destroy_host('host-id')
        manager.client.destroyVirtualMachine.assert_called_with({'id': 'host-id'})

    def test_restore_host(self):
        manager = cloudstack.CloudStackManager(self.config)
        manager.client = mock.Mock()
        manager.restore_host('host-id')
        manager.client.make_request.assert_called_with('restoreVirtualMachine',
                                                       {'virtualmachineid': 'host-id'},
                                                       response_key='restorevmresponse')



# Copyright 2014 hm authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import requests

from hm import config


_managers = {}
_expected_methods = ['create_host', 'destroy_host']


def register(name, cls):
    for m in _expected_methods:
        if not getattr(cls, m, None):
            raise InvalidManager("Expected method '{}' not found in {}".format(m, cls))
    _managers[name] = cls


def by_name(name, conf=None):
    return _managers[name](conf)


class BaseManager(object):
    def __init__(self, conf):
        self.config = conf or {}

    def get_conf(self, name, default=config.undefined):
        return config.get_config(name, default, self.config)

    def get_user_data(self):
        data = self.get_conf("USER_DATA_TXT", None)
        if data:
            return data
        url = self.get_conf("USER_DATA_URL", None)
        if not url:
            return None
        rsp = requests.get(url)
        if rsp.status_code < 200 or rsp.status_code >= 400:
            raise config.MissConfigurationError(
                "invalid user data response from {}: {} - {}".format(url, rsp.status_code, rsp.text))
        return rsp.text


class InvalidManager(Exception):
    pass



# Copyright 2014 hm authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import pymongo

from hm import config
from hm.model import host, load_balancer


class MongoDBStorage(object):
    hosts_collection = "hosts"
    lb_collection = "load_balancers"

    def __init__(self, conf=None):
        self.config = conf
        self.mongo_uri = config.get_config('DBAAS_MONGODB_ENDPOINT', None, conf)
        if not self.mongo_uri:
            self.mongo_uri = config.get_config('MONGO_URI', 'mongodb://localhost:27017/', conf)
        client = pymongo.MongoClient(self.mongo_uri)
        try:
            self.db = client.get_default_database()
            self.mongo_database = self.db.name
        except pymongo.errors.ConfigurationError:
            self.mongo_database = config.get_config('MONGO_DATABASE', 'host_manager', conf)
            self.db = client[self.mongo_database]

    def store_host(self, h):
        self._hosts_collection().insert(h.to_json())

    def remove_host(self, id):
        self._hosts_collection().remove({'_id': id})

    def find_host(self, id):
        h_data = self._hosts_collection().find_one({'_id': id})
        return host.Host.from_dict(h_data, conf=self.config)

    def list_hosts(self, filters):
        host_data_list = self._hosts_collection().find(filters) or []
        return [host.Host.from_dict(h_data, conf=self.config) for h_data in host_data_list]

    def store_load_balancer(self, lb):
        self._lb_collection().insert(lb.to_json())

    def remove_load_balancer(self, name):
        self._lb_collection().remove({'_id': name})

    def find_load_balancer(self, name):
        lb_data = self._lb_collection().find_one(name)
        return load_balancer.LoadBalancer.from_dict(lb_data, conf=self.config)

    def list_load_balancers(self, filters):
        lb_data_list = self._lb_collection().find(filters) or []
        return [load_balancer.LoadBalancer.from_dict(lb_data, conf=self.config) for lb_data in lb_data_list]

    def add_host_to_load_balancer(self, name, h):
        self._lb_collection().update({'_id': name}, {'$push': {'hosts': h.to_json()}})

    def remove_host_from_load_balancer(self, name, h):
        self._lb_collection().update({'_id': name}, {'$pull': {'hosts': {'_id': h.id}}})

    def _hosts_collection(self):
        return self.db[self.hosts_collection]

    def _lb_collection(self):
        return self.db[self.lb_collection]



# Copyright 2014 hm authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import sys

from hm import lb_managers, log
from hm.model import load_balancer
from hm.iaas.cloudstack_client import CloudStack, AsyncJobError


class CloudstackLB(lb_managers.BaseLBManager):

    def __init__(self, config=None):
        super(CloudstackLB, self).__init__(config)
        url = self.get_conf("CLOUDSTACK_API_URL")
        key = self.get_conf("CLOUDSTACK_API_KEY")
        secret_key = self.get_conf("CLOUDSTACK_SECRET_KEY")
        self.cs_client = CloudStack(url, key, secret_key)
        self.async_max_tries = int(self.get_conf("CLOUDSTACK_MAX_TRIES", 100))
        self.project_id = self.get_conf("CLOUDSTACK_PROJECT_ID", None)
        self.associate_ip_command = self.get_conf("CLOUDSTACK_LB_ASSOCIATE_IP_COMMAND", "associateIpAddress")
        self.disassociate_ip_command = self.get_conf("CLOUDSTACK_LB_DISASSOCIATE_IP_COMMAND",
                                                     "disassociateIpAddress")
        self.assign_network_command = self.get_conf("CLOUDSTACK_LB_ASSIGN_NETWORK_COMMAND", None)
        self.lb_network_id = self.get_conf("CLOUDSTACK_LB_NETWORK_ID")
        self.lb_zone_id = self.get_conf("CLOUDSTACK_LB_ZONE_ID", None)
        self.lb_vpc_id = self.get_conf("CLOUDSTACK_LB_VPC_ID", None)
        self.lb_environment_id = self.get_conf("CLOUDSTACK_LB_ENVIRONMENT_ID", None)
        self.lb_algorithm = self.get_conf("CLOUDSTACK_LB_ALGORITHM", "leastconn")
        self.lb_port_mapping = self.get_conf("CLOUDSTACK_LB_PORT_MAPPING")
        self.lb_open_firewall = self.get_conf("CLOUDSTACK_LB_OPEN_FIREWALL", "false")
        self.lb_healthcheck = self.get_conf("CLOUDSTACK_LB_HEALTHCHECK", None)
        self.lb_network_index = int(self.get_conf("CLOUDSTACK_LB_NETWORK_INDEX", 0))
        self.lb_domain = self.get_conf("CLOUDSTACK_LB_DOMAIN", None)
        self.lb_cache_group = self.get_conf("CLOUDSTACK_LB_CACHE_GROUP", None)

    def create_load_balancer(self, name):
        ip_id = self._associate_ip()
        try:
            lb_id, address = self._create_lb_rule(ip_id, name)
            self._assign_lb_additional_networks(lb_id)
            try:
                self._create_lb_hc(lb_id)
            except:
                exc_info = sys.exc_info()
                try:
                    self._delete_lb_rule(lb_id, self.project_id)
                except:
                    log.exception('error in rollback trying to delete lb rule')
                raise exc_info[0], exc_info[1], exc_info[2]

            return load_balancer.LoadBalancer(
                lb_id, name, address,
                project_id=self.project_id,
                ip_id=ip_id)
        except:
            exc_info = sys.exc_info()
            try:
                self._dissociate_ip(ip_id, self.project_id)
            except:
                log.exception('error in rollback trying to dissociate ip')
            raise exc_info[0], exc_info[1], exc_info[2]

    def destroy_load_balancer(self, lb):
        self._delete_lb_rule(lb.id, lb.project_id)
        self._dissociate_ip(lb.ip_id, lb.project_id)

    def attach_real(self, lb, host):
        list_params = {
            "id": host.id,
        }
        network_params = {
            "id": lb.id,
        }
        assign_params = {
            'id': lb.id,
            'virtualmachineids': host.id,
        }
        if hasattr(lb, 'project_id'):
            list_params["projectid"] = lb.project_id
            network_params["projectid"] = lb.project_id
            assign_params['projectid'] = lb.project_id
        if self.assign_network_command:
            vms = self.cs_client.listVirtualMachines(list_params)["virtualmachine"]
            nic = vms[0]["nic"][self.lb_network_index]
            network_params["networkids"] = nic["networkid"]
            net_rsp = self.cs_client.make_request(self.assign_network_command, network_params)
            try:
                self._wait_if_jobid(net_rsp)
            except AsyncJobError:
                log.exception('ignored error assigning network to lb')
        rsp = self.cs_client.assignToLoadBalancerRule(assign_params)
        self._wait_if_jobid(rsp)

    def detach_real(self, lb, host):
        params = {
            'id': lb.id,
            'virtualmachineids': host.id,
        }
        if hasattr(lb, 'project_id'):
            params['projectid'] = lb.project_id
        rsp = self.cs_client.removeFromLoadBalancerRule(params)
        self._wait_if_jobid(rsp)

    def _associate_ip(self):
        ip_params = {
            'networkid': self.lb_network_id
        }
        if self.project_id:
            ip_params['projectid'] = self.project_id
        if self.lb_zone_id:
            ip_params['zoneid'] = self.lb_zone_id
        if self.lb_vpc_id:
            ip_params['vpcid'] = self.lb_vpc_id
        if self.lb_environment_id:
            ip_params['lbenvironmentid'] = self.lb_environment_id
        ip_rsp = self.cs_client.make_request(self.associate_ip_command,
                                             ip_params,
                                             response_key="associateipaddressresponse")
        self._wait_if_jobid(ip_rsp)
        return ip_rsp['id']

    def _create_lb_rule(self, ip_id, name):
        public, private, additional = self._slit_ports()
        if self.lb_domain:
            name = '{}.{}'.format(name, self.lb_domain)
        lb_params = {
            'networkid': self.lb_network_id,
            'algorithm': self.lb_algorithm,
            'publicport': public,
            'privateport': private,
            'openfirewall': self.lb_open_firewall,
            'publicipid': ip_id,
            'name': name,
        }
        if additional:
            lb_params['additionalportmap'] = additional
        if self.project_id:
            lb_params['projectid'] = self.project_id
        if self.lb_cache_group:
            lb_params['cache'] = self.lb_cache_group
        lb_rsp = self.cs_client.createLoadBalancerRule(lb_params)
        result = self._wait_if_jobid(lb_rsp)
        return lb_rsp['id'], result['loadbalancer']['publicip']

    def _assign_lb_additional_networks(self, lb_id):
        network_ids = []
        i = 0
        while True:
            try:
                id = self.get_conf("CLOUDSTACK_LB_NETWORK_ID_%s" % i)
                i += 1
                if not id:
                    break
                if id == self.lb_network_id:
                    continue
                network_ids.append(id)
            except:
                break
        if not network_ids:
            return
        assign_networks_params = {
            "projectid": self.project_id,
            "id": lb_id,
            "networkids": ",".join(network_ids),
        }
        lb_rsp = self.cs_client.assignNetworksToLoadBalancerRule(assign_networks_params)
        self._wait_if_jobid(lb_rsp)

    def _create_lb_hc(self, lb_id):
        if not self.lb_healthcheck:
            return
        hc_params = {
            'lbruleid': lb_id,
            'pingpath': self.lb_healthcheck,
        }
        if self.project_id:
            hc_params['projectid'] = self.project_id
        hc_rsp = self.cs_client.createLBHealthCheckPolicy(hc_params)
        self._wait_if_jobid(hc_rsp)

    def _slit_ports(self):
        ports = self.lb_port_mapping.split(',')
        if len(ports) == 0:
            raise Exception("no port mapping defined in CLOUDSTACK_LB_PORT_MAPPING")
        mapping_parts = ports[0].split(':')
        if len(mapping_parts) != 2:
            raise Exception("invalid port mapping format in CLOUDSTACK_LB_PORT_MAPPING")
        return mapping_parts[0], mapping_parts[1], ','.join(ports[1:])

    def _wait_if_jobid(self, rsp):
        if 'jobid' not in rsp:
            return rsp
        job_result = self.cs_client.wait_for_job(rsp['jobid'], self.async_max_tries)
        return job_result['jobresult']

    def _delete_lb_rule(self, lb_id, project_id):
        lb_params = {
            'id': lb_id,
        }
        if project_id:
            lb_params['projectid'] = project_id
        lb_rsp = self.cs_client.deleteLoadBalancerRule(lb_params)
        self._wait_if_jobid(lb_rsp)

    def _dissociate_ip(self, ip_id, project_id):
        ip_params = {
            'id': ip_id,
        }
        if project_id:
            ip_params['projectid'] = project_id
        ip_rsp = self.cs_client.make_request(self.disassociate_ip_command, ip_params)
        self._wait_if_jobid(ip_rsp)


lb_managers.register('cloudstack', CloudstackLB)



# Copyright 2014 hm authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.


def storage(conf=None):
    from hm.storage import MongoDBStorage
    return MongoDBStorage(conf)


class BaseModel(object):

    def storage(self):
        return storage(self.config)



# Copyright 2014 hm authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import unittest

import pymongo.errors

from hm import lb_managers, storage
from hm.model.host import Host
from hm.model.load_balancer import LoadBalancer
from mock import patch, call


class FakeManager(lb_managers.BaseLBManager):
    def __init__(self, config=None):
        super(FakeManager, self).__init__(config)

    def create_load_balancer(self, name):
        id = self.get_conf('LB_ID')
        return LoadBalancer(id, name, 'xxx.host', extra='something')

    def destroy_load_balancer(self, lb):
        if lb.id == "explode":
            raise Exception("failure to destroy")

    def attach_real(self, lb, host):
        pass

    def detach_real(self, lb, host):
        pass

lb_managers.register('fake', FakeManager)


class LoadBalancerTestCase(unittest.TestCase):

    def setUp(self):
        mongo_stor = storage.MongoDBStorage()
        mongo_stor._lb_collection().remove()

    def test_create(self):
        conf = {'LB_ID': 'xxx'}
        lb = LoadBalancer.create('fake', 'my-lb', conf=conf)
        self.assertEqual(lb.id, 'xxx')
        self.assertEqual(lb.name, 'my-lb')
        self.assertEqual(lb.manager, 'fake')
        self.assertEqual(lb.extra, 'something')
        self.assertEqual(lb.config, conf)
        db_lb = LoadBalancer.find('my-lb', conf=conf)
        self.assertEqual(db_lb.id, 'xxx')
        self.assertEqual(db_lb.name, 'my-lb')
        self.assertEqual(db_lb.manager, 'fake')
        self.assertEqual(db_lb.extra, 'something')
        self.assertEqual(db_lb.config, conf)

    def test_create_duplicated(self):
        lb = LoadBalancer.create('fake', 'my-lb', {'LB_ID': 'xxx'})
        self.assertEqual(lb.id, 'xxx')
        self.assertEqual(lb.name, 'my-lb')
        with self.assertRaises(pymongo.errors.DuplicateKeyError):
            LoadBalancer.create('fake', 'my-lb', {'LB_ID': 'xxx'})

    def test_destroy(self):
        LoadBalancer.create('fake', 'my-lb', {'LB_ID': 'xxx'})
        db_lb = LoadBalancer.find('my-lb')
        self.assertEqual(db_lb.id, 'xxx')
        db_lb.destroy()
        db_lb = LoadBalancer.find('my-lb')
        self.assertIsNone(db_lb)

    @patch("hm.log.error")
    def test_destroy_ignores_manager_exception(self, log):
        LoadBalancer.create('fake', 'my-lb', {'LB_ID': 'explode'})
        db_lb = LoadBalancer.find('my-lb')
        self.assertEqual(db_lb.id, 'explode')
        db_lb.destroy()
        self.assertEqual(log.call_args, call("Error trying to destroy load balancer name: 'my-lb' "
                                             "id: 'explode' in 'fake': failure to destroy"))
        db_lb = LoadBalancer.find('my-lb')
        self.assertIsNone(db_lb)

    def test_add_host(self):
        h1 = Host('x', 'x.me.com')
        h2 = Host('y', 'y.me.com')
        conf = {'LB_ID': 'explode'}
        lb = LoadBalancer.create('fake', 'my-lb', conf)
        lb.add_host(h1)
        lb.add_host(h2)
        self.assertItemsEqual(lb.hosts, [h1, h2])
        db_lb = LoadBalancer.find('my-lb', conf)
        self.assertEqual(db_lb.hosts[0].config, conf)
        self.assertEqual(db_lb.hosts[1].config, conf)
        self.assertItemsEqual([h.to_json() for h in db_lb.hosts], [h1.to_json(), h2.to_json()])

    def test_remove_host(self):
        h1 = Host('x', 'x.me.com')
        h2 = Host('y', 'y.me.com')
        lb = LoadBalancer.create('fake', 'my-lb', {'LB_ID': 'explode'})
        lb.add_host(h1)
        lb.add_host(h2)
        lb.remove_host(h1)
        self.assertItemsEqual(lb.hosts, [h2])
        db_lb = LoadBalancer.find('my-lb')
        self.assertItemsEqual([h.to_json() for h in db_lb.hosts], [h2.to_json()])



# Copyright 2014 hm authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import collections

from hm import managers, log, model, config


class Host(model.BaseModel):

    def __init__(self, id, dns_name, conf=None, alternative_id=0, **kwargs):
        self.id = id
        self.dns_name = dns_name
        self.manager = None
        self.extra_args = set()
        self.config = conf
        self.alternative_id = alternative_id
        for k, v in kwargs.items():
            self.extra_args.add(k)
            setattr(self, k, v)

    def to_json(self):
        obj = {
            '_id': self.id,
            'dns_name': self.dns_name,
            'manager': self.manager,
            'alternative_id': self.alternative_id,
            'group': getattr(self, 'group', None),
        }
        for key in self.extra_args:
            obj[key] = getattr(self, key)
        return obj

    @classmethod
    def from_dict(cls, dict, conf=None):
        if dict is None:
            return None
        dict['id'] = dict['_id']
        del dict['_id']
        dict['conf'] = conf
        return Host(**dict)

    @classmethod
    def create(cls, manager_name, group, conf=None):
        manager = managers.by_name(manager_name, conf)
        alternative_id = cls._current_group_alternate(group, conf)
        host = manager.create_host(name=group, alternative_id=alternative_id)
        host.manager = manager_name
        host.group = group
        host.config = conf
        model.storage(conf).store_host(host)
        return host

    @classmethod
    def find(cls, id, conf=None):
        return model.storage(conf).find_host(id)

    @classmethod
    def list(cls, filters=None, conf=None):
        return model.storage(conf).list_hosts(filters)

    def destroy(self):
        manager = managers.by_name(self.manager, self.config)
        try:
            manager.destroy_host(self.id)
        except Exception as e:
            log.error("Error trying to destroy host '{}' in '{}': {}".format(self.id, self.manager, e))
        self.storage().remove_host(self.id)

    def restore(self):
        manager = managers.by_name(self.manager, self.config)
        try:
            manager.restore_host(self.id)
        except Exception as e:
            log.error("Error trying to restore host '{}' in '{}': {}".format(self.id, self.manager, e))
            raise e

    @classmethod
    def _current_group_alternate(cls, group, conf=None):
        alternates_count = int(config.get_config("HM_ALTERNATIVE_CONFIG_COUNT", 1, conf))
        hosts = model.storage(conf).list_hosts({'group': group})
        alterantives_map = collections.defaultdict(int)
        for host in hosts:
            alterantives_map[host.alternative_id] += 1
        min_alt_id = 0
        min_alt_count = None
        for i in xrange(alternates_count):
            if min_alt_count is None or alterantives_map[i] < min_alt_count:
                min_alt_id = i
                min_alt_count = alterantives_map[i]
        return min_alt_id



# Copyright 2014 hm authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import logging


_log = logging.getLogger('tsuru.hm')


def debug(*args):
    _log.debug(*args)


def error(*args):
    _log.error(*args)


def exception(*args):
    _log.exception(*args)


def set_handler(handler):
    _log.removeHandler(_default_handler)
    _log.addHandler(handler)

_default_handler = logging.StreamHandler()
_default_handler.setLevel(logging.INFO)
_log.addHandler(_default_handler)



# coding: utf-8
import unittest

import mock
from networkapiclient.exception import IpNaoExisteError

from hm.lb_managers import networkapi_cloudstack
from hm.model import load_balancer, host


class NetworkApiCloudstackLBTestCase(unittest.TestCase):

    def setUp(self):
        self.conf = {
            'CLOUDSTACK_API_URL': 'http://localhost',
            'CLOUDSTACK_API_KEY': 'key',
            'CLOUDSTACK_SECRET_KEY': 'secret',
            'CLOUDSTACK_PROJECT_ID': 'proj-123',
            'CLOUDSTACK_VIP_NETWORK_INDEX': '1',

            'NETWORKAPI_ENDPOINT': 'http://networkapi.host',
            'NETWORKAPI_USER': 'tsuru',
            'NETWORKAPI_PASSWORD': 'secret',
            'NETWORKAPI_CLIENTE_TXT': 'client',
            'NETWORKAPI_FINALIDADE_TXT': 'destination',
            'NETWORKAPI_AMBIENTE_P44_TXT': 'environment',

            'VIP_HEALTHCHECK': 'GET /',
            'VIP_HEALTHCHECK_EXPECT': '55',
            'VIP_METHOD_BAL': 'brincation_uite_me',
            'VIP_PERSISTENCE': 'wat',
            'VIP_CACHE': 'powerful cache',
            'VIP_MAXCONN': '1500',
            'VIP_BUSINESS_AREA': 'business-area',
            'VIP_SERVICE_NAME': 'service-name',
            'VIP_PORT_MAPPING': '80:8080',
        }

    def test_init(self):
        manager = networkapi_cloudstack.NetworkApiCloudstackLB(self.conf)
        self.assertEqual(manager.cs_client.api_url, 'http://localhost')
        self.assertEqual(manager.cs_client.api_key, 'key')
        self.assertEqual(manager.cs_client.secret, 'secret')
        self.assertEqual(manager.create_project_id, 'proj-123')
        self.assertEqual(manager.networkapi_endpoint, 'http://networkapi.host')
        self.assertEqual(manager.networkapi_user, 'tsuru')
        self.assertEqual(manager.networkapi_password, 'secret')
        self.assertEqual(manager.vip_network_index, 1)
        self.assertEqual(manager.vip_config.environment_p44, 'environment')
        self.assertEqual(manager.vip_config.client, 'client')
        self.assertEqual(manager.vip_config.finality, 'destination')
        self.assertEqual(manager.vip_config.healthcheck, 'GET /')
        self.assertEqual(manager.vip_config.healthcheck_expect, '55')
        self.assertEqual(manager.vip_config.lb_method, 'brincation_uite_me')
        self.assertEqual(manager.vip_config.cache, 'powerful cache')
        self.assertEqual(manager.vip_config.maxconn, '1500')
        self.assertEqual(manager.vip_config.business_area, 'business-area')
        self.assertEqual(manager.vip_config.service_name, 'service-name')
        self.assertEqual(manager.vip_config.port_mapping, ["80:8080"])

    def test_init_multiple_ports(self):
        self.conf.update({
            'VIP_PORT_MAPPING': '80:8080,443:8081'
        })
        manager = networkapi_cloudstack.NetworkApiCloudstackLB(self.conf)
        self.assertEqual(manager.vip_config.port_mapping, ["80:8080", "443:8081"])

    @mock.patch("hm.lb_managers.networkapi_cloudstack.log")
    @mock.patch("networkapiclient.EnvironmentVIP.EnvironmentVIP")
    @mock.patch("networkapiclient.Ip.Ip")
    @mock.patch("networkapiclient.Vip.Vip")
    def test_create_load_balancer(self, Vip, Ip, EnvironmentVIP, logger):
        client_evip = mock.Mock()
        client_evip.search.return_value = {"environment_vip": {"id": 500}}
        EnvironmentVIP.return_value = client_evip
        client_ip = mock.Mock()
        client_ip.get_available_ip4_for_vip.return_value = {"ip": {"id": 303, "oct4": "7",
                                                                   "oct2": "168", "oct3": "1",
                                                                   "oct1": "192"}}
        Ip.return_value = client_ip
        client_vip = mock.Mock()
        client_vip.add.return_value = {"requisicao_vip": {"id": 27}}
        Vip.return_value = client_vip

        manager = networkapi_cloudstack.NetworkApiCloudstackLB(self.conf)
        lb = manager.create_load_balancer("tsuru")
        self.assertEqual(27, lb.id)
        self.assertEqual(303, lb.ip_id)
        self.assertEqual("192.168.1.7", lb.address)
        self.assertEqual("tsuru", lb.name)
        self.assertEqual("proj-123", lb.project_id)

        EnvironmentVIP.assert_called_with(self.conf["NETWORKAPI_ENDPOINT"], self.conf["NETWORKAPI_USER"],
                                          self.conf["NETWORKAPI_PASSWORD"])
        Ip.assert_called_with(self.conf["NETWORKAPI_ENDPOINT"], self.conf["NETWORKAPI_USER"],
                              self.conf["NETWORKAPI_PASSWORD"])
        Vip.assert_called_with(self.conf["NETWORKAPI_ENDPOINT"], self.conf["NETWORKAPI_USER"],
                               self.conf["NETWORKAPI_PASSWORD"])
        client_evip.search.assert_called_with(ambiente_p44_txt=self.conf["NETWORKAPI_AMBIENTE_P44_TXT"],
                                              cliente_txt=self.conf["NETWORKAPI_CLIENTE_TXT"],
                                              finalidade_txt=self.conf["NETWORKAPI_FINALIDADE_TXT"])
        client_ip.get_available_ip4_for_vip.assert_called_with(500, u"tsuru hm tsuru")
        client_vip.add.assert_called_with(id_ipv4=303, id_ipv6=None,
                                          id_healthcheck_expect=self.conf["VIP_HEALTHCHECK_EXPECT"],
                                          finality=self.conf["NETWORKAPI_FINALIDADE_TXT"],
                                          client=self.conf["NETWORKAPI_CLIENTE_TXT"],
                                          environment=self.conf["NETWORKAPI_AMBIENTE_P44_TXT"],
                                          cache=self.conf["VIP_CACHE"],
                                          method_bal=self.conf["VIP_METHOD_BAL"],
                                          persistence=self.conf["VIP_PERSISTENCE"],
                                          healthcheck_type=u"HTTP",
                                          timeout=u"5",
                                          healthcheck=self.conf["VIP_HEALTHCHECK"],
                                          host="tsuru.hm.tsuru",
                                          maxcon=self.conf["VIP_MAXCONN"],
                                          areanegocio=self.conf["VIP_BUSINESS_AREA"],
                                          nome_servico=self.conf["VIP_SERVICE_NAME"],
                                          reals=[], reals_prioritys=[],
                                          reals_weights=[], ports=["80:8080"], l7_filter=None)
        client_vip.validate.assert_called_with(27)
        client_vip.criar.assert_called_with(27)
        logger.debug.assert_called_with(u"VIP request 27 successfully created.")

    @mock.patch("hm.lb_managers.networkapi_cloudstack.CloudStack")
    @mock.patch("networkapiclient.EnvironmentVIP.EnvironmentVIP")
    @mock.patch("networkapiclient.Ip.Ip")
    @mock.patch("networkapiclient.Vip.Vip")
    def test_create_vip_failure(self, Vip, Ip, EnvironmentVIP, CloudStack):
        client_evip = mock.Mock()
        client_evip.search.return_value = {"environment_vip": {"id": 500}}
        EnvironmentVIP.return_value = client_evip
        client_ip = mock.Mock()
        client_ip.get_available_ip4_for_vip.return_value = {"ip": {"id": 303}}
        Ip.return_value = client_ip
        client_vip = mock.Mock()
        client_vip.add.side_effect = Exception("Cannot create vip")
        Vip.return_value = client_vip
        manager = networkapi_cloudstack.NetworkApiCloudstackLB(self.conf)
        with self.assertRaises(Exception) as cm:
            manager.create_load_balancer("tsuru")
        exc = cm.exception
        self.assertEqual(("Cannot create vip",), exc.args)
        client_evip.search.assert_called_with(ambiente_p44_txt=self.conf["NETWORKAPI_AMBIENTE_P44_TXT"],
                                              cliente_txt=self.conf["NETWORKAPI_CLIENTE_TXT"],
                                              finalidade_txt=self.conf["NETWORKAPI_FINALIDADE_TXT"])
        client_ip.get_available_ip4_for_vip.assert_called_with(500, u"tsuru hm tsuru")
        client_ip.delete_ip4.assert_called_with(303)

    @mock.patch("hm.lb_managers.networkapi_cloudstack.CloudStack")
    @mock.patch("networkapiclient.EnvironmentVIP.EnvironmentVIP")
    @mock.patch("networkapiclient.Ip.Ip")
    @mock.patch("networkapiclient.Vip.Vip")
    def test_create_vip_failure_on_validate(self, Vip, Ip, EnvironmentVIP, CloudStack):
        client_evip = mock.Mock()
        client_evip.search.return_value = {"environment_vip": {"id": 500}}
        EnvironmentVIP.return_value = client_evip
        client_ip = mock.Mock()
        client_ip.get_available_ip4_for_vip.return_value = {"ip": {"id": 303}}
        Ip.return_value = client_ip
        client_vip = mock.Mock()
        client_vip.add.return_value = {"requisicao_vip": {"id": 27}}
        client_vip.validate.side_effect = Exception("Cannot validate vip")
        Vip.return_value = client_vip
        manager = networkapi_cloudstack.NetworkApiCloudstackLB(self.conf)
        with self.assertRaises(Exception) as cm:
            manager.create_load_balancer("tsuru")
        exc = cm.exception
        self.assertEqual(("Cannot validate vip",), exc.args)
        client_evip.search.assert_called_with(ambiente_p44_txt=self.conf["NETWORKAPI_AMBIENTE_P44_TXT"],
                                              cliente_txt=self.conf["NETWORKAPI_CLIENTE_TXT"],
                                              finalidade_txt=self.conf["NETWORKAPI_FINALIDADE_TXT"])
        client_ip.get_available_ip4_for_vip.assert_called_with(500, u"tsuru hm tsuru")
        client_ip.delete_ip4.assert_called_with(303)
        client_vip.remove_script.assert_called_with(27)
        client_vip.remover.assert_called_with(27)

    @mock.patch("hm.lb_managers.networkapi_cloudstack.log")
    @mock.patch("hm.lb_managers.networkapi_cloudstack.CloudStack")
    @mock.patch("networkapiclient.EnvironmentVIP.EnvironmentVIP")
    @mock.patch("networkapiclient.Ip.Ip")
    @mock.patch("networkapiclient.Vip.Vip")
    def test_create_vip_failure_on_remove(self, Vip, Ip, EnvironmentVIP, CloudStack, logger):
        client_evip = mock.Mock()
        client_evip.search.return_value = {"environment_vip": {"id": 500}}
        EnvironmentVIP.return_value = client_evip
        client_ip = mock.Mock()
        client_ip.get_available_ip4_for_vip.return_value = {"ip": {"id": 303}}
        Ip.return_value = client_ip
        client_vip = mock.Mock()
        client_vip.add.return_value = {"requisicao_vip": {"id": 27}}
        client_vip.validate.side_effect = Exception("Cannot validate vip")
        client_vip.remover.side_effect = Exception("Cannot remove vip")
        Vip.return_value = client_vip
        manager = networkapi_cloudstack.NetworkApiCloudstackLB(self.conf)
        with self.assertRaises(Exception) as cm:
            manager.create_load_balancer("tsuru")
        exc = cm.exception
        self.assertEqual(("Cannot validate vip",), exc.args)
        client_evip.search.assert_called_with(ambiente_p44_txt=self.conf["NETWORKAPI_AMBIENTE_P44_TXT"],
                                              cliente_txt=self.conf["NETWORKAPI_CLIENTE_TXT"],
                                              finalidade_txt=self.conf["NETWORKAPI_FINALIDADE_TXT"])
        client_ip.get_available_ip4_for_vip.assert_called_with(500, u"tsuru hm tsuru")
        client_ip.delete_ip4.assert_called_with(303)
        client_vip.remove_script.assert_called_with(27)
        client_vip.remover.assert_called_with(27)
        logger.error.assert_called_with("Failed to remove the VIP 27: Cannot remove vip")

    @mock.patch("hm.lb_managers.networkapi_cloudstack.CloudStack")
    @mock.patch("networkapiclient.Vip.Vip")
    @mock.patch("networkapiclient.Ip.Ip")
    def test_destroy_load_balancer(self, Ip, Vip, CloudStack):
        client_vip = mock.Mock()
        Vip.return_value = client_vip
        client_ip = mock.Mock()
        Ip.return_value = client_ip
        cloudstack_client = CloudStack.return_value

        lb = load_balancer.LoadBalancer(
            "404", "myapp", "192.168.1.1",
            ip_id="303",
            project_id="project_id-x"
        )
        manager = networkapi_cloudstack.NetworkApiCloudstackLB(self.conf)
        manager.destroy_load_balancer(lb)
        Vip.assert_called_with(self.conf["NETWORKAPI_ENDPOINT"], self.conf["NETWORKAPI_USER"],
                               self.conf["NETWORKAPI_PASSWORD"])
        client_vip.remove_script.assert_called_with("404")
        client_vip.remover.assert_called_with("404")
        Ip.assert_called_with(self.conf["NETWORKAPI_ENDPOINT"], self.conf["NETWORKAPI_USER"],
                              self.conf["NETWORKAPI_PASSWORD"])
        client_ip.delete_ip4.assert_called_with("303")
        data = {"projectid": "project_id-x", "vipid": "404"}
        cloudstack_client.removeGloboNetworkVip.assert_called_with(data)

    @mock.patch("hm.lb_managers.networkapi_cloudstack.CloudStack")
    @mock.patch("networkapiclient.Vip.Vip")
    @mock.patch("networkapiclient.Ip.Ip")
    def test_remove_vip_ip_not_found(self, Ip, Vip, CloudStack):
        client_vip = mock.Mock()
        Vip.return_value = client_vip
        client_ip = mock.Mock()
        client_ip.delete_ip4.side_effect = IpNaoExisteError("303")
        Ip.return_value = client_ip
        lb = load_balancer.LoadBalancer(
            "404", "myapp", "192.168.1.1",
            ip_id="303",
            project_id="project_id-x"
        )
        manager = networkapi_cloudstack.NetworkApiCloudstackLB(self.conf)
        manager.destroy_load_balancer(lb)
        Vip.assert_called_with(self.conf["NETWORKAPI_ENDPOINT"], self.conf["NETWORKAPI_USER"],
                               self.conf["NETWORKAPI_PASSWORD"])
        client_vip.remove_script.assert_called_with("404")
        client_vip.remover.assert_called_with("404")
        Ip.assert_called_with(self.conf["NETWORKAPI_ENDPOINT"], self.conf["NETWORKAPI_USER"],
                              self.conf["NETWORKAPI_PASSWORD"])
        client_ip.delete_ip4.assert_called_with("303")

    @mock.patch("hm.lb_managers.networkapi_cloudstack.CloudStack")
    def test_attach_real(self, CloudStack):
        self.conf.update({'CLOUDSTACK_VIP_NETWORK_INDEX': '0'})
        vms = {"virtualmachine": [{"id": "abc123", "nic": [{"id": "def456", "networkid": "netid1"}]}]}
        CloudStack.return_value = cloudstack_client = mock.Mock()
        cloudstack_client.listVirtualMachines.return_value = vms

        manager = networkapi_cloudstack.NetworkApiCloudstackLB(self.conf)
        lb = load_balancer.LoadBalancer(
            "500", "myapp", "192.168.1.1",
            ip_id="303",
            project_id=None,
        )
        h = host.Host('abc123', 'name.host')
        manager.attach_real(lb, h)
        list_data = {"id": "abc123"}
        cloudstack_client.listVirtualMachines.assert_called_with(list_data)
        net_data = {"networkid": "netid1", "vipid": "500"}
        cloudstack_client.addGloboNetworkVipToAccount.assert_called_with(net_data)
        assoc_data = {"nicid": "def456", "vipid": "500"}
        cloudstack_client.associateGloboNetworkRealToVip.assert_called_with(assoc_data)
        CloudStack.assert_called_with(
            self.conf["CLOUDSTACK_API_URL"],
            self.conf["CLOUDSTACK_API_KEY"],
            self.conf["CLOUDSTACK_SECRET_KEY"])

    @mock.patch("hm.lb_managers.networkapi_cloudstack.CloudStack")
    def test_attach_real_with_project_id(self, CloudStack):
        self.conf.update({'CLOUDSTACK_VIP_NETWORK_INDEX': '0'})
        vms = {"virtualmachine": [{"id": "abc123", "nic": [{"id": "def456", "networkid": "netid1"}]}]}
        CloudStack.return_value = cloudstack_client = mock.Mock()
        cloudstack_client.listVirtualMachines.return_value = vms

        manager = networkapi_cloudstack.NetworkApiCloudstackLB(self.conf)
        lb = load_balancer.LoadBalancer(
            "500", "myapp", "192.168.1.1",
            ip_id="303",
            project_id="project_id-x"
        )
        h = host.Host('abc123', 'name.host')
        manager.attach_real(lb, h)
        list_data = {"id": "abc123", "projectid": "project_id-x"}
        cloudstack_client.listVirtualMachines.assert_called_with(list_data)
        net_data = {"networkid": "netid1", "vipid": "500", "projectid": "project_id-x"}
        cloudstack_client.addGloboNetworkVipToAccount.assert_called_with(net_data)
        assoc_data = {"nicid": "def456", "vipid": "500", "projectid": "project_id-x"}
        cloudstack_client.associateGloboNetworkRealToVip.assert_called_with(assoc_data)
        CloudStack.assert_called_with(
            self.conf["CLOUDSTACK_API_URL"],
            self.conf["CLOUDSTACK_API_KEY"],
            self.conf["CLOUDSTACK_SECRET_KEY"])

    @mock.patch("hm.lb_managers.networkapi_cloudstack.CloudStack")
    def test_attach_real_with_network_index(self, CloudStack):
        vms = {"virtualmachine": [{"id": "abc123", "nic": [
            {"id": "def456", "networkid": "netid1"}, {"id": "second-id", "networkid": "netid2"}]}]}
        CloudStack.return_value = cloudstack_client = mock.Mock()
        cloudstack_client.listVirtualMachines.return_value = vms

        manager = networkapi_cloudstack.NetworkApiCloudstackLB(self.conf)
        lb = load_balancer.LoadBalancer(
            "500", "myapp", "192.168.1.1",
            ip_id="303",
            project_id=None,
        )
        h = host.Host('abc123', 'name.host')
        manager.attach_real(lb, h)
        list_data = {"id": "abc123"}
        cloudstack_client.listVirtualMachines.assert_called_with(list_data)
        net_data = {"networkid": "netid2", "vipid": "500"}
        cloudstack_client.addGloboNetworkVipToAccount.assert_called_with(net_data)
        assoc_data = {"nicid": "second-id", "vipid": "500"}
        cloudstack_client.associateGloboNetworkRealToVip.assert_called_with(assoc_data)
        CloudStack.assert_called_with(
            self.conf["CLOUDSTACK_API_URL"],
            self.conf["CLOUDSTACK_API_KEY"],
            self.conf["CLOUDSTACK_SECRET_KEY"])

    @mock.patch("hm.lb_managers.networkapi_cloudstack.CloudStack")
    def test_detach_real(self, CloudStack):
        self.conf.update({'CLOUDSTACK_VIP_NETWORK_INDEX': '0'})
        vms = {"virtualmachine": [{"id": "abc123", "nic": [{"id": "def456", "networkid": "netid1"}]}]}
        CloudStack.return_value = cloudstack_client = mock.Mock()
        cloudstack_client.listVirtualMachines.return_value = vms
        manager = networkapi_cloudstack.NetworkApiCloudstackLB(self.conf)
        lb = load_balancer.LoadBalancer(
            "500", "myapp", "192.168.1.1",
            ip_id="303",
            project_id=None
        )
        h = host.Host('abc123', 'name.host')
        manager.detach_real(lb, h)
        list_data = {"id": "abc123"}
        cloudstack_client.listVirtualMachines.assert_called_with(list_data)
        assoc_data = {"nicid": "def456", "vipid": "500"}
        cloudstack_client.disassociateGloboNetworkRealFromVip.assert_called_with(assoc_data)
        CloudStack.assert_called_with(
            self.conf["CLOUDSTACK_API_URL"],
            self.conf["CLOUDSTACK_API_KEY"],
            self.conf["CLOUDSTACK_SECRET_KEY"])

    @mock.patch("hm.lb_managers.networkapi_cloudstack.CloudStack")
    def test_detach_real_with_project_id(self, CloudStack):
        self.conf.update({'CLOUDSTACK_VIP_NETWORK_INDEX': '0'})
        vms = {"virtualmachine": [{"id": "abc123", "nic": [{"id": "def456", "networkid": "netid1"}]}]}
        CloudStack.return_value = cloudstack_client = mock.Mock()
        cloudstack_client.listVirtualMachines.return_value = vms
        manager = networkapi_cloudstack.NetworkApiCloudstackLB(self.conf)
        lb = load_balancer.LoadBalancer(
            "500", "myapp", "192.168.1.1",
            ip_id="303",
            project_id="pid-xxx"
        )
        h = host.Host('abc123', 'name.host')
        manager.detach_real(lb, h)
        list_data = {"id": "abc123", "projectid": "pid-xxx"}
        cloudstack_client.listVirtualMachines.assert_called_with(list_data)
        assoc_data = {"nicid": "def456", "vipid": "500", "projectid": "pid-xxx"}
        cloudstack_client.disassociateGloboNetworkRealFromVip.assert_called_with(assoc_data)
        CloudStack.assert_called_with(
            self.conf["CLOUDSTACK_API_URL"],
            self.conf["CLOUDSTACK_API_KEY"],
            self.conf["CLOUDSTACK_SECRET_KEY"])



# -*- coding: utf-8 -*-

# Copyright 2014 hm authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.
import codecs

from setuptools import setup, find_packages

from hm import __version__

README = codecs.open('README.rst', encoding='utf-8').read()

setup(
    name="tsuru-hm",
    url="https://github.com/tsuru/hm",
    version=__version__,
    description="Host manager library for tsuru PaaS services",
    long_description=README,
    author="Tsuru",
    author_email="tsuru@corp.globo.com",
    classifiers=[
        "Programming Language :: Python :: 2.7",
    ],
    packages=find_packages(exclude=["docs", "tests"]),
    include_package_data=True,
    install_requires=[
        "boto==2.25.0",
        "pymongo==2.6.3",
        "requests==2.4.3",
        "GloboNetworkAPI==0.2.2",
    ],
    extras_require={
        'tests': [
            "mock==1.0.1",
            "flake8==2.1.0",
            "coverage==3.7.1",
            "freezegun==0.2.8",
        ]
    },
)



# Copyright 2012 Kelcey Damage & Kraig Amador
# Source: http://goo.gl/KQUeMd

import base64
import hmac
import hashlib
import json
import urllib
import time

from hm import log


JOB_PENDING = 0
JOB_SUCCESS = 1
JOB_ERROR = 2


class CloudStack(object):

    def __init__(self, api_url, api_key, secret):
        self.api_url = api_url
        self.api_key = api_key
        self.secret = secret

    def encode_user_data(self, data):
        return base64.b64encode(data)

    def request(self, args):
        args["apiKey"] = self.api_key
        self.params = []
        self.sig_params = []
        self._sort_request(args)
        self._create_signature()
        self._build_post_request()

    def _sort_request(self, args):
        keys = sorted(args.keys())
        for key in keys:
            self.sig_params.append(
                key.lower() + "=" + urllib.quote_plus(args[key]).lower().replace('+', '%20'))
            self.params.append(key + "=" + urllib.quote_plus(args[key]))

    def _create_signature(self):
        self.query = "&".join(self.params)
        self.sig_query = "&".join(self.sig_params)
        digest = hmac.new(self.secret, msg=self.sig_query,
                          digestmod=hashlib.sha1).digest()
        self.signature = base64.b64encode(digest)

    def _build_post_request(self):
        self.query += "&signature=" + urllib.quote_plus(self.signature)
        self.value = self.api_url + "?" + self.query

    def __getattr__(self, name):
        def handler(*args, **kwargs):
            if kwargs:
                return self.make_request(name, kwargs)
            return self.make_request(name, args[0])
        return handler

    def _http_get(self, url):
        response = urllib.urlopen(url)
        return response.read()

    def make_request(self, command, args, response_key=None):
        args["response"] = "json"
        args["command"] = command
        self.request(args)
        data = self._http_get(self.value)
        key = command.lower() + "response"
        rsp_data = json.loads(data)
        response = rsp_data.get(key)
        if response is None and response_key:
            key = response_key
            response = rsp_data.get(key)
        if response is None or 'errorcode' in response:
            raise InvalidResponse("Invalid response running '{} {}': {}".format(command, args, data))
        log.debug("GET {}: {}".format(self.value, response))
        return response

    def wait_for_job(self, job_id, max_tries):
        status = JOB_PENDING
        tries = 0
        result = None
        while tries < max_tries:
            result = self.queryAsyncJobResult({"jobid": job_id})
            status = result["jobstatus"]
            if status != JOB_PENDING:
                break
            time.sleep(1)
            tries += 1
        if status == JOB_PENDING:
            raise MaxTryWaitingForJobError(max_tries, job_id)
        if status == JOB_ERROR:
            raise AsyncJobError("async job error: {}".format(result))
        return result


class AsyncJobError(Exception):
    pass


class InvalidResponse(Exception):
    pass


class MaxTryWaitingForJobError(Exception):

    def __init__(self, max_tries, job_id):
        self.max_tries = max_tries
        self.job_id = job_id
        msg = "exceeded {0} tries waiting for job {1}".format(max_tries, job_id)
        super(MaxTryWaitingForJobError, self).__init__(msg)
